'use strict';

/**
 * @file
 * @author Andrey Galkin <andrey@futoin.org>
 *
 *
 * Copyright 2014-2017 FutoIn Project (https://futoin.org)
 * Copyright 2014-2017 Andrey Galkin <andrey@futoin.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ISync = require('./ISync');
var Mutex = require('./Mutex');
var Throttle = require('./Throttle');

/**
 * Limiter - complex processing limit for AsyncSteps
 */

var Limiter = function (_ISync) {
    _inherits(Limiter, _ISync);

    /**
     * C-tor
     * @param {object} [options={}] - option map
     * @param {integer} [options.concurrent=1]  - maximum concurrent flows
     * @param {integer} [options.max_queue=0] - maximum queued
     * @param {integer} [options.rate=1]  - maximum entries in period
     * @param {integer} [options.period_ms=1000]  - period length
     * @param {integer} [options.burst=0]  - maximum queue for rate limiting
     */
    function Limiter() {
        var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

        _classCallCheck(this, Limiter);

        var _this = _possibleConstructorReturn(this, (Limiter.__proto__ || Object.getPrototypeOf(Limiter)).call(this));

        _this._mutex = new Mutex(options.concurrent || 1, options.max_queue || 0);
        _this._throttle = new Throttle(options.rate || 1, options.period_ms || 1000, options.burst || 0);
        return _this;
    }

    _createClass(Limiter, [{
        key: 'sync',
        value: function sync(as, step, onerror) {
            var _this2 = this;

            as.sync(this._mutex, function (as) {
                as._root._next_args = as._call_args;
                as.sync(_this2._throttle, step, onerror);
            });
        }
    }]);

    return Limiter;
}(ISync);

module.exports = Limiter;
//# sourceMappingURL=Limiter.js.map
