'use strict';

/**
 * @file
 * @author Andrey Galkin <andrey@futoin.org>
 *
 *
 * Copyright 2014-2017 FutoIn Project (https://futoin.org)
 * Copyright 2014-2017 Andrey Galkin <andrey@futoin.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ISync = require('./ISync');

var _require = require('./Errors'),
    DefenseRejected = _require.DefenseRejected;

var _require2 = require('./lib/common'),
    prev_queue = _require2.prev_queue;

var mtx_sync = function mtx_sync(asp, mtx, step, on_error) {
    var root = asp._root;
    var release_step = function release_step(asi) {
        mtx._release(root);
        root._handle_success(asi._call_args);
    };

    asp._on_cancel = mtx._release_handler;
    root._next_args = asp._call_args;

    if (mtx._lock(asp, root)) {
        asp._queue = [[step, on_error], [release_step, undefined]];
    } else {
        prev_queue(root).unshift([step, on_error], [release_step, undefined]);
    }
};

/**
 * Mutual exclusion mechanism for AsyncSteps
 */

var Mutex = function (_ISync) {
    _inherits(Mutex, _ISync);

    /**
     * C-tor
     * @param {integer} [max=1] - maximum number of simultaneous critical section entries
     * @param {integer} [max_queue=null] - limit queue length, if set
     */
    function Mutex() {
        var max = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
        var max_queue = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

        _classCallCheck(this, Mutex);

        var _this = _possibleConstructorReturn(this, (Mutex.__proto__ || Object.getPrototypeOf(Mutex)).call(this));

        _this._max = max;
        _this._locked = 0;
        _this._owners = new WeakMap();
        _this._queue = [];
        _this._max_queue = max_queue;
        _this._release_handler = function (asi) {
            _this._release(asi._root);
        };
        return _this;
    }

    _createClass(Mutex, [{
        key: '_lock',
        value: function _lock(asi, key) {
            var owners = this._owners;
            var owned = owners.get(key);

            if (owned) {
                owners.set(key, owned + 1);
                return true;
            } else if (this._locked >= this._max) {
                var queue = this._queue;
                var max_queue = this._max_queue;

                if (max_queue !== null && queue.length >= max_queue) {
                    asi.error(DefenseRejected, 'Mutex queue limit');
                }

                queue.push(asi);
                return false;
            } else {
                this._locked += 1;
                owners.set(key, 1);
                return true;
            }
        }
    }, {
        key: '_release',
        value: function _release(key) {
            var owners = this._owners;
            var owned = owners.get(key);

            if (owned) {
                if (owned > 1) {
                    owners.set(key, owned - 1);
                    return;
                }

                owners.delete(key);

                this._locked -= 1;
                var queue = this._queue;

                while (queue.length) {
                    var other_as = queue.shift();

                    if (other_as.state) {
                        var other_root = other_as._root;
                        this._lock(other_as, other_root);
                        other_root._handle_success(other_root._next_args);
                        break;
                    }
                }
            } else {
                var idx = this._queue.indexOf(key);

                if (idx >= 0) {
                    this._queue.splice(idx, 1);
                }
            }
        }
    }, {
        key: 'sync',
        value: function sync(as, step, onerror) {
            var _this2 = this;

            as.add(function (as) {
                mtx_sync(as, _this2, step, onerror);
            });
        }
    }]);

    return Mutex;
}(ISync);

module.exports = Mutex;
//# sourceMappingURL=Mutex.js.map
