'use strict';

/**
 * @file
 * @author Andrey Galkin <andrey@futoin.org>
 *
 *
 * Copyright 2014-2017 FutoIn Project (https://futoin.org)
 * Copyright 2014-2017 Andrey Galkin <andrey@futoin.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var ISync = require('./ISync');

var _require = require('./Errors'),
    DefenseRejected = _require.DefenseRejected;

var _require2 = require('./lib/common'),
    prev_queue = _require2.prev_queue;

var throttle_sync = function throttle_sync(asp, throttle, step, on_error) {
    var root = asp._root;

    if (throttle._lock(asp)) {
        asp._on_error = on_error;
        step.apply(undefined, [asp].concat(_toConsumableArray(asp._call_args)));
    } else {
        asp._on_cancel = throttle._cancel_handler;
        root._next_args = asp._call_args;

        prev_queue(root).unshift([step, on_error]);
    }
};

/**
 * Throttling for AsyncSteps
 */

var Throttle = function (_ISync) {
    _inherits(Throttle, _ISync);

    /**
     * C-tor
     * @param {integer} [max=1] - maximum number of simultaneous critical section entries
     * @param {intger} [period_ms=1000] - time period in milliseconds
     * @param {integer} [max_queue=null] - limit queue length, if set
     */
    function Throttle(max) {
        var period_ms = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1e3;
        var max_queue = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

        _classCallCheck(this, Throttle);

        var _this = _possibleConstructorReturn(this, (Throttle.__proto__ || Object.getPrototypeOf(Throttle)).call(this));

        _this._max = max;
        _this._current = 0;
        var queue = _this._queue = [];
        _this._timer = null;
        _this._period_ms = period_ms;
        _this._max_queue = max_queue;
        _this._cancel_handler = function (asi) {
            var idx = queue.indexOf(asi);

            if (idx >= 0) {
                queue.splice(idx, 1);
            }
        };
        return _this;
    }

    _createClass(Throttle, [{
        key: '_lock',
        value: function _lock(asi) {
            this._ensureTimer();

            if (this._current >= this._max) {
                var queue = this._queue;
                var max_queue = this._max_queue;

                if (max_queue !== null && queue.length >= max_queue) {
                    asi.error(DefenseRejected, 'Throttle queue limit');
                }

                queue.push(asi);
                return false;
            } else {
                this._current += 1;
                return true;
            }
        }
    }, {
        key: '_ensureTimer',
        value: function _ensureTimer() {
            var _this2 = this;

            if (!this._timer) {
                this._timer = setInterval(function () {
                    _this2._resetPeriod();
                }, this._period_ms);
            }
        }
    }, {
        key: '_resetPeriod',
        value: function _resetPeriod() {
            this._current = 0;
            var queue = this._queue;

            if (!queue.length) {
                clearInterval(this._timer);
                this._timer = null;
                return;
            }

            var max = this._max;
            var current = 0;

            while (queue.length && current < max) {
                var other_as = queue.shift();

                if (other_as.state) {
                    ++current;
                    var other_root = other_as._root;
                    other_root._handle_success(other_root._next_args);
                }
            }

            this._current = current;
        }
    }, {
        key: 'sync',
        value: function sync(as, step, onerror) {
            var _this3 = this;

            as.add(function (as) {
                throttle_sync(as, _this3, step, onerror);
            });
        }
    }]);

    return Throttle;
}(ISync);

module.exports = Throttle;
//# sourceMappingURL=Throttle.js.map
