'use strict';

// ensure it works with frozen one

Object.freeze(Object.prototype);

var chai = require('chai');

//
var $as = typeof window !== 'undefined' ? require('futoin-asyncsteps') : module.require('../lib/main-full');

var assert = chai.assert,
    expect = chai.expect;
var Limiter = $as.Limiter;

var _require = require('../lib/common'),
    MAX = _require.MAX,
    MAX_QUEUE = _require.MAX_QUEUE,
    MUTEX = _require.MUTEX,
    PERIOD_MS = _require.PERIOD_MS,
    THROTTLE = _require.THROTTLE;

describe('Limiter', function () {
    it('should have correct defaults', function () {
        var lim = new Limiter();

        expect(lim._mutex._max).to.equal(1);
        expect(lim._mutex._max_queue).to.equal(0);

        expect(lim._throttle._max).to.equal(1);
        expect(lim._throttle._period_ms).to.equal(1e3);
        expect(lim._throttle._max_queue).to.equal(0);
    });

    it('should have correct defaults', function () {
        var lim = new Limiter({
            concurrent: 12,
            max_queue: 23,
            rate: 34,
            period_ms: 1234,
            burst: 45
        });

        expect(lim._mutex._max).to.equal(12);
        expect(lim._mutex._max_queue).to.equal(23);

        expect(lim._throttle._max).to.equal(34);
        expect(lim._throttle._period_ms).to.equal(1234);
        expect(lim._throttle._max_queue).to.equal(45);
    });

    it('should handle limits', function (done) {
        if (typeof window !== 'undefined') {
            // Quite lags in browser
            done();
            return;
        }

        var lim = new Limiter({
            concurrent: 20,
            max_queue: 77,
            rate: 5,
            period_ms: 100,
            burst: 12
        });

        var as = $as();
        var p = as.parallel(function (as, err) {
            done(as.state.last_exception || err || 'Fail');
        });
        var passed = 0;
        var rejected = 0;

        for (var i = 0; i < 100; ++i) {
            p.add(function (as) {
                as.add(function (as) {
                    return as.sync(lim, function (as) {
                        passed += 1;
                    });
                }, function (as, err) {
                    if (err === 'DefenseRejected') {
                        rejected += 1;
                        as.success();
                    }
                });
            });
        }

        as.execute();

        setTimeout(function () {
            expect(passed).to.equal(5);
            expect(rejected).to.equal(83);
        }, 75);

        setTimeout(function () {
            expect(passed).to.equal(10);
            expect(rejected).to.equal(83);
        }, 150);

        setTimeout(function () {
            expect(passed).to.equal(5 + 12);
            expect(rejected).to.equal(83);
            as.cancel();
            done();
        }, 350);
    });

    it('should handle re-entrancy & success params', function (done) {
        var lim = new Limiter({
            concurrent: 1,
            max_queue: 77,
            rate: 5,
            period_ms: 100,
            burst: 12
        });

        var _loop = function _loop(i) {
            var as = $as();

            as.add(function (as) {
                return as.success('MyArgs');
            });
            as.sync(lim, function (as, arg) {
                expect(arg).to.equal('MyArgs');

                as.sync(lim, function (as) {
                    return as.error('Wrong');
                }, function (as, err) {
                    if (err === 'Wrong') {
                        as.error('OK');
                    }
                });
            }, function (as, err) {
                as.success(err);
            });
            as.add(function (as, err) {
                if (err === 'OK') {
                    if (i) {
                        done();
                    }
                } else {
                    done('Fail');
                }
            });
            as.execute();
        };

        for (var i = 0; i < 2; ++i) {
            _loop(i);
        }
    });
});
//# sourceMappingURL=limitertest.js.map
